package clo.brp.raven.i4;

import javax.swing.*;
import java.awt.*;
import java.awt.Image;

import clo.brp.ui.WindowUtils;

/**
 * JWindow splash screeen with a progress bar. Centered on the screen.
 *
 * @author Jason Rohrer
 * @version 7-14-2000//7-14-2000
 */
public class SplashScreen extends JWindow {

    JProgressBar mProgressBar;
    JLabel mExtraMessageLabel = new JLabel("");

    static final int WINDOW_WIDTH = 400;
    static final int WINDOW_HEIGHT = 300;

    static final int PROGRESS_MIN = 0;
    static final int PROGRESS_MAX = 1000;

    /**
     * Constructs a splash screen, specifying a string to be displayed
     *
     * @param inString text to be displayed on the splash screen
     */
    public SplashScreen( String inString ) {
        setSize( WINDOW_WIDTH, WINDOW_HEIGHT );

        getContentPane().setLayout( new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ) );

        JPanel stringPanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );

        JLabel stringLabel = new JLabel( inString );

        stringPanel.add( stringLabel );

        JPanel progressPanel = new JPanel();
        setupProgressPanel( progressPanel );

        JPanel extraMessagePanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        extraMessagePanel.add( mExtraMessageLabel );

        getContentPane().add( stringPanel );
        getContentPane().add( progressPanel );
        getContentPane().add( extraMessagePanel );

        WindowUtils.centerWindow( this );

        show();
    }

    /**
     * Constructs a splash screen, specifying an image to be displayed
     *
     * @param inImage image to be displayed on the splash screen
     */
    public SplashScreen( Image inImage ) {
        setSize( WINDOW_WIDTH, WINDOW_HEIGHT );

        getContentPane().setLayout( new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ) );

        JPanel imagePanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );

        int imageWidth = inImage.getWidth( this );
        int imageHeight = inImage.getHeight( this );

        imagePanel.setSize( WINDOW_WIDTH, WINDOW_HEIGHT );
        imagePanel.setPreferredSize( new Dimension( WINDOW_WIDTH, imageHeight ) );

        Graphics panelGraphics = imagePanel.getGraphics();

        panelGraphics.drawImage( inImage, ( WINDOW_WIDTH - imageWidth ) / 2, 0, this );

        JPanel progressPanel = new JPanel();
        setupProgressPanel( progressPanel );

        JPanel extraMessagePanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        extraMessagePanel.add( mExtraMessageLabel );

        getContentPane().add( imagePanel );
        getContentPane().add( progressPanel );
        getContentPane().add( extraMessagePanel );

        WindowUtils.centerWindow( this );

        show();
    }


    /**
     * Sets fraction of progess bar filled.
     *
     * @param inProgress fraction of progress bar filled in [0.0, 1.0]
     */
    public void setProgress( float inProgress ) {
        mProgressBar.setValue( (int)( inProgress * ( PROGRESS_MAX - PROGRESS_MIN ) ) );
    }


    /**
     * Sets message to be displayed on progress bar
     *
     * @param inMessage message displayed on progress bar
     */
    public void setProgressMessage( String inMessage ) {
        mProgressBar.setString( inMessage );
    }


    /**
     * Sets message to be displayed below the progress bar
     *
     * @param inExtraMessage message to be displayed below the progress bar
     */
    public void setExtraMessage( String inExtraMessage ) {
        mExtraMessageLabel.setText( inExtraMessage );
    }


    /**
     * Method used by both constructors to setup the progress bar panel
     */
    private void setupProgressPanel( JPanel inPanel ) {
        inPanel.setLayout( new FlowLayout( FlowLayout.CENTER ) );

        mProgressBar = new JProgressBar( PROGRESS_MIN, PROGRESS_MAX );

        mProgressBar.setStringPainted( true );
        mProgressBar.setString( "" );

        mProgressBar.setValue( PROGRESS_MIN );

        inPanel.add( mProgressBar );
    }

}