// Jason Rohrer
// ScrollTablePanel.java

/**
*
*	GUI component that contains a scrolling multi-column table of GUI components
*
*	Created 5-9-2000
*	Mods:
*/

package konspire.client;

import javax.swing.*;

import javax.swing.table.*;

import javax.swing.table.DefaultTableModel;

import java.awt.Component;
import java.awt.FlowLayout;	

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import konspire.common.Vector;

public class ScrollTablePanel extends EtchedTitledJPanel {

	private String title;
	private String columnTitles[];
	private Class columnClasses[];

	Vector rows = new Vector();		// vector of TableRow wrappers in same order as table rows

	int numCols;

	protected JTable table;
	private GUITableModel tableModel;
	
	private JScrollPane tableScrollPane;
	
	private boolean beenSetup = false;
	
	protected ScrollTablePanel() {
		// constructor called by subclasses when custom setup needed
		}
	
	public ScrollTablePanel( String titl, String[] colTitles, Class[] colClasses ) {
		setup( titl, colTitles, colClasses );
		}

	protected void setup( String titl, String[] colTitles, Class[] colClasses ) {
		
		title = titl;
		setTitle( title );
		
		columnTitles = colTitles;
		columnClasses = colClasses;
		
		numCols = columnTitles.length;
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	
		
		tableModel = new GUITableModel( columnTitles, 0); 
		table = new JTable( tableModel );
		
		TableCellRenderer defaultRenderer = table.getDefaultRenderer( Component.class );
		JTableGUIRenderer guiRenderer = new JTableGUIRenderer(defaultRenderer);
		
		table.setDefaultRenderer( Component.class, guiRenderer );
		
		table.setCellSelectionEnabled( false );
		table.setRowSelectionAllowed( false );
		table.setColumnSelectionAllowed( false );
		
		JTableGUIMouseListener tableListener = new JTableGUIMouseListener();
		
		table.addMouseListener( tableListener );
		
		tableScrollPane = new JScrollPane( table );
		
		add( tableScrollPane );
		
		beenSetup = true;
		}
	
	
	// add a row of objects to table
	// object types must match with fixed classes defined for each table column
	// returns a TableRow wrapper corresponding to this row (for row removal)
	public TableRow addRow( Object [] oArray ) {
		Object b = oArray[0];
		// make sure class order matches fixed class order
		int maxInd = oArray.length;
		if( maxInd > numCols ) {
			maxInd = numCols;
			}
		for( int i=0; i<maxInd; i++ ) {
			if( (oArray[i]).getClass() != columnClasses[i] ) {
				System.out.println("Object array can't be added to table:  Type mismatch with fixed column types.");
				return null;
				}
			}
		TableRow rowWrapper = new TableRow( oArray );
		rows.addElement( rowWrapper );
			
		// else all classes match
		tableModel.addRow( oArray );
		//return tableModel.getRowCount() - 1;
		return rowWrapper;
		}
	
	// remove a row from table
	public boolean removeRow( TableRow rowWrapper ) {
		if( rows.contains( rowWrapper ) ) {
			int index = rows.indexOf( rowWrapper ); 
			
			tableModel.removeRow( index );
			rows.removeElementAt( index );
			
			return true;
			}
		return false;
		}
	
	// remove all rows from table
	public void removeAllRows() {
		int lastRow = tableModel.getRowCount() -1;
		for( int i=lastRow; i>=0; i-- ) {
			tableModel.removeRow( i );
			}
		rows.removeAllElements();
		}
	
	
	// table model for tables with embedded gui components
	private class GUITableModel extends DefaultTableModel {
		 
		 public GUITableModel(Object[] columnNames, int numRows) {
			super(columnNames, numRows);
			} 
		 
		 public Class getColumnClass(int c) {
		 	// FIXED 6-13-2000
		 //	return getValueAt(0, c).getClass();
			if( beenSetup ) {
				return columnClasses[ c ];
				}
			else {
				return Object.class;
				}
		
			}
		
		public boolean isCellEditable(int row, int column) {
			return false;
			}	
		}
		
	
	class JTableGUIRenderer implements TableCellRenderer {
		private TableCellRenderer defaultRenderer;

		public JTableGUIRenderer(TableCellRenderer renderer) {
			defaultRenderer = renderer;
			}

		public Component getTableCellRendererComponent(JTable table, Object value,
							 boolean isSelected,
							 boolean hasFocus,
							 int row, int column) {
			if(value instanceof Component) {
				return (Component)value;
				}
			return defaultRenderer.getTableCellRendererComponent( 
					table, value, isSelected, hasFocus, row, column);
			}
		}
		
		
	class JTableGUIMouseListener implements MouseListener {
		
		private void passEventToComponent( MouseEvent e ) {

			TableColumnModel columnModel = table.getColumnModel();
			int column = columnModel.getColumnIndexAtX(e.getX());
			int row    = e.getY() / table.getRowHeight();
			Object value;
			Component comp;
			MouseEvent compEvent;

			if(row >= table.getRowCount() || row < 0 ||
			   column >= table.getColumnCount() || column < 0) {
				return;
				}
			value = table.getValueAt(row, column);

			if( !(value instanceof Component) ) {
			  return;
			  }
		  
			comp = (Component)value;

			compEvent =
			  (MouseEvent)SwingUtilities.convertMouseEvent( table, e, comp );
			comp.dispatchEvent(compEvent);

			table.repaint();
			}
	
		public void mouseClicked(MouseEvent e) {
			passEventToComponent(e);
			}

		public void mouseEntered(MouseEvent e) {
			passEventToComponent(e);
			}

		public void mouseExited(MouseEvent e) {
			passEventToComponent(e);
			}

		public void mousePressed(MouseEvent e) {
			passEventToComponent(e);
			}

		public void mouseReleased(MouseEvent e) {
			passEventToComponent(e);
			}

		}
	
	}